package types

import (
	"time"

	"gorm.io/gorm"
)

type EtiquetaTipo uint8

const (
	EtiquetaEuropea EtiquetaTipo = iota + 1
	EtiquetaBritanica
	EtiquetaAmericana
)

var (
	TipoFormato = map[EtiquetaTipo]string{
		EtiquetaEuropea:   "EU-200dpi",
		EtiquetaAmericana: "US-200dpi",
		EtiquetaBritanica: "UK-200dpi",
	}
)

type Label struct {
	Codigo    string `json:"codigo" gorm:"primaryKey"`
	Tipo      EtiquetaTipo
	Contenido string
	CreatedAt time.Time
	UpdatedAt time.Time
	DeletedAt gorm.DeletedAt `gorm:"index"`
}

func (l *Label) BeforeCreate(tx *gorm.DB) (err error) {
	l.CreatedAt = time.Now()
	l.UpdatedAt = time.Now()
	return
}

func (l *Label) BeforeUpdate(tx *gorm.DB) (err error) {
	l.UpdatedAt = time.Now()
	return
}
